from django.conf import settings
from django.core.urlresolvers import reverse
from django.core.management.base import BaseCommand

from django.contrib.sites.models import Site

from ...models import WaitingListEntry, SurveyQuestion, SurveyAnswer

from ... import trello


class Command(BaseCommand):

    help = "Export waiting list entries to Trello for use as a simple CRM"

    def export_contacts(self):
        entries = WaitingListEntry.objects.filter(trello_card_id="")
        for entry in entries:
            # 0. Title of card is email address
            title = entry.email.encode("utf-8")
            answers = []
            try:
                qs = entry.surveyinstance.answers.all().order_by("question__pk")
                for answer in qs:
                    if answer.question.kind == SurveyQuestion.BOOLEAN_FIELD or len(answer.value) > 0:
                        answers.append(answer)
            except:
                answers = []

            description = ""
            if len(answers) > 0:
                list_id = self.api.answered_surveys_list_id
                # 1. Create Markdown for Description
                for answer in answers:
                    description += "#### {0}:\n".format(answer.question.question.encode("utf-8"))
                    if answer.question.kind == SurveyQuestion.BOOLEAN_FIELD:
                        description += "> {0}\n\n".format(answer.value_boolean)
                    else:
                        description += "> {0}\n\n".format(answer.value.encode("utf-8"))
            else:
                list_id = self.api.imported_contacts_list_id
            description += "\n\n-----\n\nID: {0}\n".format(str(entry.pk))
            description += "Admin Link: {0}://{1}{2}\n".format(self.protocol, self.site.domain, reverse("admin:waitinglist_waitinglistentry_change", args=[entry.pk]))
            description += "Created At: {0}\n".format(str(entry.created))

            # 2. Create the card
            card = self.api.create_card(title, description, list_id)
            # 3. Store the id
            entry.trello_card_id = card["id"]
            entry.save()
            print "Exported {0} onto Card {1}".format(entry.email.encode("utf-8"), entry.trello_card_id)

    def export_answers(self):
        questions = SurveyQuestion.objects.filter(trello_list_id="")
        for question in questions:
            question.trello_list_id = self.api.setup_board(question.question)
            question.save()
        answers = SurveyAnswer.objects.filter(trello_card_id="")
        for answer in answers:
            description = "Entry: {0}\n".format(self.api.card_short_url(answer.instance.entry.trello_card_id))
            if answer.question.kind == SurveyQuestion.BOOLEAN_FIELD:
                title = str(answer.value_boolean)
            else:
                title = answer.value.encode("utf-8")
            if title.strip() != "":
                card = self.api.create_card(title, description, answer.question.trello_list_id)
                answer.trello_card_id = card["id"]
                answer.save()
                print "Exported Answer {0} on Card {1}".format(answer.pk, answer.trello_card_id)

    def handle(self, *args, **options):
        self.site = Site.objects.get_current()
        self.protocol = getattr(settings, "DEFAULT_HTTP_PROTOCOL", "http")
        self.api = trello.Api()
        self.export_contacts()
        self.export_answers()



from django.conf import settings

import requests


class Api(object):

    def __init__(self):
        self.base_url = "https://api.trello.com"
        self.token = getattr(settings, "WAITINGLIST_TRELLO_TOKEN", None)
        self.key = getattr(settings, "WAITINGLIST_TRELLO_KEY", None)
        self.org_slug = getattr(settings, "WAITINGLIST_TRELLO_ORG_SLUG", None)
        self.board_short_id = getattr(settings, "WAITINGLIST_TRELLO_BOARD_SHORT_ID", None)
        self.answered_surveys_list_name = getattr(settings, "WAITINGLIST_TRELLO_ANSWERED_SURVEY_LIST_NAME", "Answered Surveys")
        self.imported_contacts_list_name = getattr(settings, "WAITINGLIST_TRELLO_IMPORTED_CONTACTS_LIST_NAME", "Imported Contacts")
        self.imported_answers_list_name = getattr(settings, "WAITINGLIST_TRELLO_IMPORTED_ANSWERS_LIST_NAME", "Imported Answered")
        self._answered_surveys_list_id = None
        self._imported_contacts_list_id = None
        self._board_id = None
        self._org_id = None

    @property
    def org_id(self):
        if not self._org_id:
            url = "/1/organizations/{0}?token={1}&key={2}".format(self.org_slug, self.token, self.key)
            print url
            r = requests.get("{0}{1}".format(self.base_url, url))
            print r.status_code
            self._org_id = requests.get("{0}{1}".format(self.base_url, url)).json()["id"]
        return self._org_id

    @property
    def board_id(self):
        if not self._board_id:
            url = "/1/organizations/{0}/boards/?token={1}&key={2}".format(self.org_id, self.token, self.key)
            boards = requests.get("{0}{1}".format(self.base_url, url)).json()
            for board in boards:
                if board["shortLink"] == self.board_short_id:
                    self._board_id = board["id"]
                    break
        return self._board_id

    def lists(self, board_id):
        url = "/1/boards/{0}/lists?token={1}&key={2}".format(self.board_id, self.token, self.key)
        return requests.get("{0}{1}".format(self.base_url, url)).json()

    def create_list(self, name, board_id):
        url = "/1/lists?token={0}&key={1}".format(self.token, self.key)
        return requests.post("{0}{1}".format(self.base_url, url), data={
            "name": name,
            "idBoard": board_id
        }).json()

    def card_short_url(self, card_id):
        url = "/1/cards/{0}/shortUrl?token={1}&key={2}".format(card_id, self.token, self.key)
        return requests.get("{0}{1}".format(self.base_url, url)).json()["_value"]

    def _get_or_create_list(self, name):
        list_id = None
        for lst in self.lists(self.board_id):
            if lst["name"] == name:
                list_id = lst["id"]
        if list_id is None:
            list_id = self.create_list(name, self.board_id)["id"]
        return list_id

    @property
    def answered_surveys_list_id(self):
        if not self._answered_surveys_list_id:
            self._answered_surveys_list_id = self._get_or_create_list(self.answered_surveys_list_name)
        return self._answered_surveys_list_id

    @property
    def imported_contacts_list_id(self):
        if not self._imported_contacts_list_id:
            self._imported_contacts_list_id = self._get_or_create_list(self.imported_contacts_list_name)
        return self._imported_contacts_list_id

    def create_card(self, title, description, list_id):
        url = "/1/cards?token={0}&key={1}".format(self.token, self.key)
        return requests.post("{0}{1}".format(self.base_url, url), data={"name": title, "desc": description, "idList": list_id}).json()

    def delete_card(self, card_id):
        url = "/1/cards/{2}?token={0}&key={1}".format(self.token, self.key, card_id)
        return requests.delete("{0}{1}".format(self.base_url, url))

    def setup_board(self, name):
        url = "/1/boards/?token={0}&key={1}".format(self.token, self.key)
        board_data = requests.post("{0}{1}".format(self.base_url, url), data={
            "name": name.encode("utf-8"),
            "prefs_permissionLevel": "org",
            "prefs_selfJoin": "true",
            "idOrganization": self.org_id
        }).json()
        return self.create_list(self.imported_answers_list_name, board_data["id"])["id"]
